unit Graphe;

interface

uses windows, sysUtils, graphics, classes, printers,
     Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs,
     jmmUtil, math, tvcLib_TLB;

const
        IntSansSignif = -1;
        MaxVecteur = 5000;
        CouleurAxe : Tcolor = clGray;
        precisionAff = 4;
        voieX = 0;
        voieT = 0;
        voieA = 1; // CH1
        voieB = 2; // CH2
        voieF = 0;
        voieGain = 1;
        voiePhase = 2;
        voieMax = 2;

type
        TCalculIncertitude = (ciNone,ciFreq,ciTemps,ciTension,ciTensionEff);
        indiceVecteur = 0..MaxVecteur;
        vecteur = array[indiceVecteur] of double;
        VecteurOsc = array[indiceVecteur] of double;
        indiceVoie = 0..voieMax;

        TvoieRecord = record
                mini,maxi : double;
                Nom,Command,Unite,Signif : string;
                Active,Positive : boolean;
                CalculIncertitude : TCalculIncertitude;
                NumVoie : integer;
        end;

        TvoieOsc = class
                mini,maxi : double;
                a,b : double; // entier = rel*a+b
                Nom,Unite,Signif : string;
                Active : boolean;
                couleur : Tcolor;
                valeur : vecteurOsc;
                Function GrandeurToEcran(i : integer) : integer;
                constructor create;
        end;

        TvoieTDS = class
                mini,maxi : double;
                Nom,Command,Unite,Signif : string;
                Commentaire : String;
                Active,Positive : boolean;
                CalculIncertitude : TCalculIncertitude;
                NumVoie : integer;
                constructor create;
        end;

        Tvoie = class(TvoieTDS)
                a,b : double; // entier = rel*a+b
                valeurCourante,valeurAff : double;
                incertitudeCourante : double;
                valeur,incertitude : vecteur;
                AxeLog : boolean;
                AxedB : boolean;
                LabelVoie : Tlabel;
                GBvoie : TgroupBox;
                couleur : Tcolor;
                constructor create;
                Function GrandeurToEcran(x : double) : integer;
                Procedure SetIncertitude;
                Function StrValeur(nombre : double) : String;
                Function AffVoltmetre : String;
           end;

        Tgraphe = class
                 dimPoint : integer;
                 Canvas : Tcanvas;
                 voie : array[indiceVoie] of Tvoie;
                 NbrePoints : integer;
                 DerniereVoie : indiceVoie;
                 LimiteCourbe : Trect;
                 PointCourantOK : boolean;
                 avecAxeY : boolean;
                 Constructor Create;
                 Procedure DrawG(Arect : Trect);
                 Procedure EnregistrePointCourant;
                 Procedure AffichePointCourant(curseur : boolean);
                 Procedure TraceCroix(xr,yr,dxr,dyr : double;v : indiceVoie;Curseur : boolean);
                 procedure DrawAxis;
                 procedure SaveData(const NomFichier : String);
                 destructor Destroy; override;
                 procedure LineG(x1,y1,x2,y2 : integer);
                 procedure tri;
       end;

       TgrapheOsc = class
                 Canvas : Tcanvas;
                 voie : array[indiceVoie] of TvoieOsc;
                 NbrePoints : integer;
                 Constructor Create;
                 Procedure DrawG(Arect : Trect);
                 destructor Destroy; override;
       end;

const
    CouleurVoieDefaut : array[indiceVoie] of Tcolor = (clBlack,clRed,clGreen);
    ChiffreEtc : TsysCharSet = ['0'..'9','+','-','e','E','.'];
    VoieDefaut : array[indiceVoie] of TvoieRecord =
    ((mini:100;maxi:10000;Nom : 'f';command : 'FREQ';
     unite : 'Hz';Signif : 'Frquence';active : true;positive : true;
     calculIncertitude : ciFreq;Numvoie : 1),
     (mini:0;maxi:5;Nom : 'Ve';command : 'CRMS';
     unite : 'V';signif : 'Tension efficace';active : true;positive : true;
     calculIncertitude : ciTensionEff;Numvoie : 1),
    (mini:0;maxi:5;Nom : 'Vs';command : 'CRMS';
     unite : 'V';signif : 'Tension efficace';active : true;positive : true;
     calculIncertitude : ciTensionEff;Numvoie : 2));

var BaseDeTemps,DeltaT : double;
    DeltaV : array[1..2] of double;

IMPLEMENTATION

constructor TvoieTDS.Create;
begin
        inherited create;
        nom := '';
        Signif := '';
        Unite := '';
        Commentaire := '';
        Command := '';
        Maxi := 1;
        Mini := 0;
        Active := false;
        Positive := true;
        CalculIncertitude := ciNone;
        NumVoie := 1;
end; // TvoieTDS.Create

constructor Tvoie.Create;
begin
        inherited create;
        A := 1;
        B := 0;
        ValeurCourante := 0;
        ValeurAff := 0;
        incertitudeCourante := 0;
        calculIncertitude := ciNone;
        NumVoie := 1;
        AxeLog := false;
        AxedB := false;
end; //  Tvoie.Create

Function Tvoie.GrandeurToEcran(x : double) : integer;
begin
     try
     if axeLog then x := log10(x);
     if axedB then x := 20*log10(x);
     result := round(A*x+B)
     except
     result := 0;
     end;
end;

Function TvoieOsc.GrandeurToEcran(i : integer) : integer;
begin
     try
     result := round(A*valeur[i]+B)
     except
     result := 0;
     end;
end;

Procedure Tgraphe.Tri;
{ Tri Shell Meyer-Baudoin p 456 }
{ origine vecteur en 0 }
var cle : double;
    i,j : integer;
    v : indiceVoie;
    SauvePoint : array[indiceVoie] of double;
Begin
    i := 1;
    while (i<NbrePoints) do begin
        for v := voieX to VoieMax do
            sauvePoint[v] := voie[v].valeur[i];
        cle := voie[voieX].valeur[i];
        j := pred(i);
        while (j>=0) and (voie[voieX].valeur[j]>cle) do begin
             for v := voieX to VoieMax do
                 voie[v].valeur[succ(j)] := voie[v].valeur[j];
             dec(j);
        end;
        for v := voieX to VoieMax do
            voie[v].valeur[succ(j)] := sauvePoint[v];
        inc(i);
    end;
end; // tri

procedure Tgraphe.EnregistrePointCourant;
var i : indiceVoie;
begin
    for i := voieX to DerniereVoie do with voie[i] do begin
        valeur[NbrePoints] := valeurAff;
        incertitude[NbrePoints] := incertitudeCourante;
    end;
    inc(NbrePoints);
    AffichePointCourant(false);
end;

procedure Tgraphe.TraceCroix(xr,yr,dxr,dyr : double;v : indiceVoie;curseur : boolean);
var x,y,dx,dy,y1,y2 : integer;
begin
        x := voie[voieX].GrandeurToEcran(xr);
        dx := dimPoint;
        y1 := voie[v].grandeurToEcran(yr-dyr);
        y2 := voie[v].grandeurToEcran(yr+dyr);
        dy := (y1-y2) div 2;
        if dy<dimPoint then dy := dimPoint;
        y := (y1+y2) div 2;
        Canvas.Pen.color := Voie[v].couleur;
        Canvas.Brush.color := Voie[v].couleur;
        if curseur
           then begin
              Canvas.MoveTo(x-3*dimPoint,y);
              Canvas.LineTo(x+3*dimPoint,y);
              Canvas.MoveTo(x,y-3*dimPoint);
              Canvas.LineTo(x,y+3*dimPoint);
           end
           else Canvas.RoundRect(x-dx,y-dy,x+dx,y+dy,1,1);
end;

procedure Tgraphe.AffichePointCourant(curseur : boolean);
var i : indiceVoie;
begin
    PointCourantOK := false;
    for i := voieX to DerniereVoie do with voie[i] do
        if isNan(valeurCourante) then exit;
    PointCourantOK := true;
    if curseur
       then canvas.pen.width := 3
       else canvas.brush.style := bsSolid;
    for i := voieX to DerniereVoie do
        voie[i].valeurAff := voie[i].valeurCourante;
    for i := voieA to DerniereVoie do
        TraceCroix(voie[voieX].valeurCourante,
                   voie[i].valeurCourante,
                   voie[voieX].incertitudeCourante,
                   voie[i].incertitudeCourante,i,curseur);
    canvas.pen.width := 1;
end;

procedure Tgraphe.DrawG(Arect : Trect);
var j : indiceVoie;
    i,largeur : integer;
begin
     LimiteCourbe := Arect;
     canvas.font.size := 10;
     canvas.Pen.Mode := pmCopy;
     largeur := canvas.TextWidth('-0.001');
     dec(LimiteCourbe.bottom,abs(canvas.font.height*2));
     inc(LimiteCourbe.top,2);
     if avecAxeY then begin
         inc(LimiteCourbe.left,largeur);
         dec(LimiteCourbe.right,largeur);
     end;
     canvas.brush.color := clWindow;
     canvas.font.color := couleurAxe;
     canvas.FillRect(canvas.clipRect);
     canvas.brush.style := bsSolid;
     with voie[voieX],LimiteCourbe do begin
        dimPoint := succ((bottom-top) div 240);
        if mini<=1 then mini := 1;
        if maxi<=mini then maxi := mini*10;
        if axeLog
           then begin
              A := (right-left)/log10(Maxi/Mini);
              B := left-log10(Mini)*A;
           end
           else begin
              A := (right-left)/(Maxi-Mini);
              B := left-Mini*A;
           end;
     end;
     canvas.Pen.style := psSolid;
     for j := voieA to DerniereVoie do with LimiteCourbe,voie[j] do begin
         if axedB
            then begin
               if mini<1e-4 then mini := 1e-4;
               if maxi<=mini then maxi := mini*10;
               A := (top-bottom)/log10(Maxi/Mini)/20;
               B := bottom-20*log10(Mini)*A;
            end
            else begin
               if maxi<=mini then maxi := mini+1;
               A := (top-bottom)/(Maxi-Mini);
               B := bottom-Mini*A;
            end;
     end;
     drawAxis;
     for i := 0 to pred(NbrePoints) do
         for j := voieA to DerniereVoie do
           TraceCroix(voie[voieX].valeur[i],
              voie[j].valeur[i],
              voie[voieX].Incertitude[i],
              voie[j].Incertitude[i],j,false);
     pointCourantOK := false;
end;

procedure TgrapheOsc.DrawG(Arect : Trect);
var j : indiceVoie;
    i : integer;
    x,y : integer;
begin
     canvas.Pen.Mode := pmCopy;
     canvas.Pen.style := psSolid;
     canvas.Pen.color := clBlack;
     canvas.brush.color := clBlack;
     canvas.brush.style := bsSolid;
     canvas.FillRect(canvas.clipRect);
     if NbrePoints<16 then exit;
     with voie[voieT],Arect do begin
          Maxi := valeur[NbrePoints-1];
          Mini := valeur[1];
          A := (right-left)/(Maxi-Mini);
          B := left-Mini*A;
     end;
     for j := voieA to voieB do with voie[j] do if Active then begin
     with Arect do begin
          if maxi<0.05 then maxi := 0.05;
          mini := -maxi;
          A := (top-bottom)/(Maxi-Mini);
          B := bottom-Mini*A;
     end;
     canvas.Pen.Color := couleur;
     x := voie[voieT].GrandeurToEcran(0);
     y := voie[j].grandeurToEcran(0);
     Canvas.MoveTo(x,y);
     for i := 1 to NbrePoints-1 do begin
        x := voie[voieT].GrandeurToEcran(i);
        y := voie[j].grandeurToEcran(i);
        Canvas.LineTo(x,y);
     end;
     end;// with voie
end;

destructor Tgraphe.destroy;
var j : indiceVoie;
begin
    for j := voieX to VoieMax do voie[j].free;
    inherited destroy;
end;

destructor TgrapheOsc.destroy;
var v : indiceVoie;
begin
    for v := voieT to VoieMax do voie[v].free;
    inherited destroy;
end;

constructor Tgraphe.Create;
var j : indiceVoie;
begin
  inherited create;
  for j := voieX to VoieMax do begin
      voie[j] := Tvoie.create;
      with voie[j] do begin
              Nom := VoieDefaut[j].nom;
              Unite := VoieDefaut[j].unite;
              Signif := VoieDefaut[j].signif;
              Command := VoieDefaut[j].command;
              Active := VoieDefaut[j].active;
              Positive := VoieDefaut[j].positive;
              CalculIncertitude := VoieDefaut[j].CalculIncertitude;
              Mini := VoieDefaut[j].mini;
              Maxi := VoieDefaut[j].maxi;
              NumVoie := VoieDefaut[j].NumVoie;
              Couleur := couleurVoieDefaut[j];
        end;
   end;
   voie[voieX].axeLog := true;
   DerniereVoie := voieA;
   NbrePoints := 0;
end;

constructor TvoieOsc.Create;
begin
       Nom := 'V';
       Unite := 'V';
       Signif := 'tension';
       Active := true;
       Mini := 0;
       Maxi := 1;
       Couleur := clBlack;
end;

constructor TgrapheOsc.Create;
var j : indiceVoie;
begin
  inherited create;
  for j := voieT to VoieMax do
      voie[j] := TvoieOsc.create;
  voie[voieA].Nom := 'V1';
  voie[voieB].Nom := 'V2';
  voie[voieT].Nom := 't';
  voie[voieA].couleur := clBlue;
  voie[voieB].couleur := clRed;
  voie[voieT].Unite := 's';
  voie[voieT].Signif := 'temps';
  NbrePoints := 0;
end;

procedure Tgraphe.LineG(x1,y1,x2,y2 : integer);
begin
     canvas.moveTo(x1,y1);
     canvas.lineTo(x2,y2);
end;

procedure Tgraphe.DrawAxis;

 procedure traceAxeY(v : indiceVoie);
 var deltaAxe,exposant : double;
     pointCourant,maxiAxe : double;

procedure SetDelta;
{ Exposant=puissance de 10 multiple de 3 de manire  avoir entre
NbreGraduations (10) et N/2.5 (4) graduations sur l'axes.
ou NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
Delta est l'intervalle entre les graduations qui commencent  PremiereGraduation }
const
    indexMax = 5;
    NbreGraduations = 10;
    coeff : array[0..indexMax] of double = (0.1,0.2,0.5,1,2,5);
var  largeur : double;
     Exponent,index : integer;
begin with voie[v] do begin
    if axedB
       then begin
          Exponent := 0;
          largeur := 20*log10(maxi/mini);
          maxiAxe := 20*log10(maxi);
       end
       else begin
          Exponent := floor(Log10(Maxi-Mini));
          largeur := (maxi-mini)*power(10,-exponent);
          maxiAxe := maxi;
       end;   
    index := 0;
    while ( (largeur/coeff[index])>=NbreGraduations ) and
                    (index<indexMax) do
                 inc(index);
    DeltaAxe := coeff[index]*power(10,exponent);
    if axedB
       then PointCourant := ceil(20*log10(mini)/deltaAxe)*deltaAxe
       else PointCourant := ceil(mini/deltaAxe)*deltaAxe;
    Index := Exponent div 3;
    if ( (Exponent mod 3) < 0) then dec(index);
  { la division euclidienne des informaticiens <> div des maths ! }
    Exposant := power(10,3*Index);
end end; { setDelta }

 const marge = 6;
 var ys,signe,xAxeY,xTexte : Integer;
     nombreAff,titre : string;
 begin with voie[v] do begin
      setDelta;
      if v=voieGain
         then begin
            xAxeY := limiteCourbe.left;
            xTexte := marge;
            signe := 1;
         end
         else begin
            xAxeY := limiteCourbe.right;
            xTexte := xAxeY+marge;
            signe := -1;
          end;
      canvas.pen.style := psSolid;
      canvas.pen.color := couleur;
      canvas.font.color := couleur;
      repeat
           if axedB
              then ys := grandeurToEcran(power(10,PointCourant/20))
              else ys := grandeurToEcran(PointCourant);
           NombreAff := formatCourt(pointCourant/exposant,'');
           canvas.TextOut(xTexte,ys-marge,NombreAff);
           LineG(xAxeY,ys,xAxeY+signe*marge,ys);
           pointCourant := pointCourant+deltaAxe;
      until ( pointCourant>=maxiAxe );
      titre := nom;
      if axedb
         then titre := titre+'dB'
         else if unite<>'' then titre := titre+'('+unite+')';
      canvas.TextOut(xTexte,limiteCourbe.top+marge,titre);
      LineG(xAxeY,limiteCourbe.top,xAxeY,limiteCourbe.bottom);
end end; // traceAxeY

procedure traceAxeLogX;
var i,j : integer;
    xs  : integer;
    intermediaire : boolean;
begin with canvas,limiteCourbe,voie[voieX] do begin
   Intermediaire := log10(Maxi/Mini) <= 4;
   pen.style := psDot;
   pen.color := CouleurAxe;
   for i := round(log10(Mini)) to round(log10(Maxi)) do begin
        xs := grandeurToEcran(power(10,i));
        LineG(xs,top,xs,bottom);
        TextOut(xs,bottom+2,formatCourt(power(10,i),Unite));
        if intermediaire then for j := 2 to 9 do begin
             xs := grandeurToEcran(power(10,i+log10(j)));
             LineG(xs,top,xs,bottom);
        end;{for j}
   end; {for i}
   pen.style := psSolid;
end end; // TraceAxeLogx

procedure traceAxeX;
var i : integer;
    xs  : integer;
    x : double;
begin with canvas,limiteCourbe,voie[voieX] do begin
   pen.color := CouleurAxe;
   pen.Style := psDot;
   for i := 1 to 4 do begin
        x := i*Maxi/5;
        xs := grandeurToEcran(x);
        LineG(xs,top,xs,bottom);
        TextOut(xs,bottom+2,formatCourt(x,unite));
   end; {for i}
   pen.Style := psSolid;
end end;// TraceAxeX

var i,y : integer;
begin with canvas,LimiteCourbe do begin
     for i := 0 to 10 do begin
          y := round(top+i*(bottom-top) div 10);
          LineG(left,y,right,y)
     end;
     if voie[voieX].axeLog then traceAxeLogX else traceAxeX;
     if avecAxeY then begin
        traceAxeY(voieGain);
        traceAxeY(voiePhase);
     end;
     TextOut(right+2*abs(font.height),bottom+2,voie[voieX].nom);
end end;// DrawAxis

procedure Tgraphe.SaveData(const NomFichier : String);
var fichier : textFile;
    formatRW3 : boolean;

Procedure EcritPage;
var j : integer;
    i : indiceVoie;
begin if formatRW3
  then begin
     writeln(fichier,SymbReg,'1 PAGE COMMENT');
     writeln(fichier,'');
     writeln(fichier,SymbReg2,NbrePoints,' VALEUR VAR');
     for j := 0 to pred(NbrePoints) do begin
         for i := voieX to DerniereVoie do
             write(fichier,FloatToStrF(voie[i].valeur[j],ffGeneral,7,1),' ');
        writeln(fichier);
     end
 end
 else begin
    for j := 0 to pred(NbrePoints) do begin
        for i := voieX to DerniereVoie do
  	     write(fichier,FloatToStrPoint(voie[i].valeur[j]),crTab);
        writeln(fichier);
    end;
end
end; // ecritPage

Procedure ecritEnTete;
var j : indiceVoie;
    N : integer;
begin if formatRW3
then begin
   N := DerniereVoie+1;
   writeln(fichier,'EVARISTE REGRESSI WINDOWS 1.0');
   writeln(fichier,symbReg,N,' NOM VAR');
   for j := voieX to DerniereVoie do
      writeln(fichier,voie[j].nom);
   writeln(fichier,symbReg,N,' UNITE VAR');
   for j := voieX to DerniereVoie do
      writeln(fichier,voie[j].unite);
   writeln(fichier,symbReg,N,' SIGNIF VAR');
   for j := voieX to DerniereVoie do
      writeln(fichier,voie[j].commentaire);
   writeln(fichier,symbReg,'1 ACQUISITION');
   writeln(fichier,Application.ExeName)
end
else begin
   writeln(fichier,'TDS200');
   for j := voieX to DerniereVoie do
      write(fichier,voie[j].nom,crTab);
   writeln(fichier);
   for j := voieX to DerniereVoie do
      write(fichier,voie[j].unite,crTab);
   writeln(fichier);
   for j := voieX to DerniereVoie do
      write(fichier,voie[j].signif,crTab);
   writeln(fichier);
end
end;// ecritEnTete

begin // saveData
    FormatRW3 := AnsiUpperCase(ExtractFileExt(NomFichier))='.RW3';
    AssignFile(fichier,NomFichier);
    Rewrite(fichier);
    ecritEnTete;
    tri;
    ecritPage;
    CloseFile(fichier);
end; // saveData

procedure Tvoie.SetIncertitude;
begin
      incertitudeCourante := 0;
      if not isNan(valeurCourante) then begin
      case calculIncertitude of
           ciNone : ;
           ciFreq : incertitudeCourante := DeltaT*sqr(valeurCourante);
           ciTemps : incertitudeCourante := DeltaT;
           ciTension : incertitudeCourante := DeltaV[NumVoie];
           ciTensionEff : incertitudeCourante := DeltaV[NumVoie]/4;
      end;
      end;
end;

Function Tvoie.AffVoltmetre : String;
begin
    if axedB
       then result := nom+'='+strValeur(20*log10(valeurCourante))+'dB'
       else result := nom+'='+strValeur(valeurCourante);
end;

Function Tvoie.StrValeur(nombre : double) : String;
var coeff : coefUnite;
    Negatif : boolean;
    tampon : String;
begin
    if isNan(nombre)
       then StrValeur := 'erreur'
       else begin
    negatif := nombre<0;
    nombre := abs(nombre);
    coeff := nulle;
    if Unite<>'' then begin
       while ( nombre>=999 ) and (coeff<infini) do begin
           nombre := nombre/1000;
           inc(coeff)
       end; { nombre<1000 ou infini }
       while ( nombre<0.999 ) and (coeff>zero) do begin
           nombre := nombre*1000;
           Dec(coeff)
       end; { nombre>=1 ou zero }
    end;
    if coeff in [infini,zero]
      then tampon := '0 '
      else begin
         tampon := FloatToStrF(nombre,ffGeneral,precisionAff,0);
         if negatif then tampon := '-' + tampon;
         tampon := tampon + ' ';
         if coeff<>nulle then tampon := tampon+symboleUnite[coeff];
      end;
      StrValeur := tampon+unite;
   end;
end;

end.

