unit Jmmutil;

interface

uses
   SysUtils, Windows, Messages, Classes, Dialogs, Forms,
   StdCtrls, Consts, math, shlObj,
   system.UITypes;

const
   enDegre: boolean = True;
   SymbReg: char = '';
   SymbReg2: char = '&';
   Tabulation = #9;

   Xinc          = 'WFMP:XIN?';
   Xzero         = 'WFMP:XZE?';
   Yinc          = 'WFMP:YMU?';
   Yoffset       = 'WFMP:YOF?';
   Yzero         = 'WFMP:YZE?';
   GetData       = 'CURV?';
   SetSourceOsc  = 'DAT:SOU CH';
   CHactive      = 'SEL:CH';
   GetTimeDiv    = 'HOR:SCA?';
   SetSourceMult = 'SOU CH';
   SetType       = 'TYP';
   GetValeurStr  = 'VAL?';
   EnTeteMesure  = 'MEASU:IMM:';

type
   CoefUnite = (zero, yocto, zepto, atto, femto, pico, nano, micro, milli,
      nulle, kilo, mega, giga, tera, peta, exa, zetta, yotta, infini);

const
   RadianVersDegre = 57.29577951308232090;
   Majuscule: TsyscharSet = ['A'..'Z'];
   Minuscule: TsyscharSet = ['a'..'z'];
   Chiffre: TsyscharSet = ['0'..'9'];
   ChiffreEtc: TsyscharSet = ['0'..'9', '+', '-', 'e', 'E', '.'];
   ChiffreWin: TsyscharSet = ['0'..'9', '+', '-', 'e', 'E', '.', ','];
   crTab      = char(vk_tab);
   crSupprArr = char(vk_back);
   crHaut     = char(vk_up);
   crBas      = char(vk_down);
   crGauche   = char(vk_left);
   crDroite   = char(vk_right);
   crHome     = char(vk_home);
   crEnd      = char(vk_end);
   crCR       = #13;
   crLF       = #10;
   crESC      = #27;
   symboleUnite: array[coefUnite] of char =
      ('0', 'y', 'z', 'a', 'f', 'p', 'n', '', 'm', ' ', 'k', 'M', 'G',
      'T', 'P', 'E', 'Z', 'Y', '');
   valUnite: array[coefUnite] of real =
      (1E-27, 1E-24, 1E-21, 1E-18, 1E-15, 1E-12, 1E-9, 1E-6, 1E-3, 1,
      1E+3, 1E+6, 1E+9, 1E+12, 1E+15, 1E+18, 1E+21, 1E+24, 1E+27);
   separateur: TsysCharSet   = [' ', crTab, ','];
   caracPrefixe: TsyscharSet =
      ['y', 'z', 'a', 'f', 'p', 'n', '', 'm', 'k', 'M', 'G', 'T', 'P', 'E'];

function FormatCourt(nombre: double; NomUnite: string): string;
function FloatToStrPoint(valeur: double): string;
function StrToFloatWin(nombre: string): double;
function StrToFloatPoint(nombre: string): double;
function IntMaths(x: double): double;
function OK(texte: String): boolean;
procedure Erreur(texte: String);
function Atan2(im, re: double): double;

var
   mesDocsDir : string;
   NomFichierIni: string;
   nomVisa: string;

implementation

Function StrToFloatWin(nombre : String) : double;
var posSeparator : integer;
begin
   if formatSettings.decimalSeparator<>'.' then begin
       posSeparator := pos('.',nombre);
       if posSeparator>0 then Nombre[posSeparator] := formatSettings.decimalSeparator;
   end
   else if formatSettings.decimalSeparator<>',' then begin
       posSeparator := pos(',',nombre);
       if posSeparator>0 then Nombre[posSeparator] := formatSettings.decimalSeparator;
   end;
   result :=  strToFloat(nombre);
end;

Function StrToFloatPoint(nombre : String) : double;
var posSeparator : integer;
begin
   if formatSettings.decimalSeparator<>'.' then begin
      posSeparator := pos('.',nombre);
      if posSeparator>0 then Nombre[posSeparator] := formatSettings.decimalSeparator;
   end;
   result :=  strToFloat(nombre);
end;

function FloatToStrPoint(valeur: double): string;
var
   nombre: string;
   posSeparator: integer;
begin
   nombre := FloatToStrF(valeur, ffGeneral, 7, 1);
   if formatSettings.decimalSeparator<>'.' then begin
      posSeparator := pos(formatSettings.decimalSeparator, nombre);
      if posSeparator > 0 then Nombre[posSeparator] := '.';
   end;
   Result := nombre;
end;

function FormatCourt(Nombre: double; NomUnite: string): string;
var
   unite:  coefUnite;
   Negatif: boolean;
   tampon: string;
begin
   if isNan(nombre)
   then result := 'erreur'
   else begin
      negatif := nombre < 0;
      nombre := abs(nombre);
      unite := nulle;
      while (nombre >= 999) and (unite < infini) do begin
         nombre := nombre / 1000;
         Inc(unite);
      end; { nombre<1000 ou infini }
      while (nombre < 0.999) and (unite > zero) do begin
         nombre := nombre * 1000;
         Dec(unite);
      end; { nombre>=1 ou zero }
      if unite in [infini, zero] then
         tampon := '0 '
      else begin
         tampon := FloatToStrF(nombre, ffGeneral, 7, 0);
         if negatif then
            tampon := '-' + tampon;
         tampon := tampon + ' ';
         if unite <> nulle then
            tampon := tampon + symboleUnite[unite];
      end;
      result := tampon + NomUnite;
   end;
end;

function IntMaths(x: double): double;
//  "vraie" partie entire
var
   zut: double;
begin
   Zut := int(x);
   if (x >= 0) or (zut = x) then
      intMaths := zut
   else
      intMaths := zut - 1;
end;

function OK(texte: String): boolean;
begin
   OK := MessageDlg(texte, mtConfirmation, [mbYes, mbNo], 0) = idYes;
end;

procedure Erreur(texte: String);
begin
   MessageDlg(texte, mtInformation, [mbOK], 0);
end;

function Atan2(im, re: double): double;
// z = re + j*im
begin
   try
      Result := arcTan(im / re);
      if re < 0 then
         if im < 0 then
            Result := Result - pi
         else
            Result := Result + pi;
   except
      on EZeroDivide do begin
         if im < 0 then
            Result := -pi / 2
         else
            Result := pi / 2;
      end;
   end;
   if EnDegre then
      Result := Result * RadianVersDegre;
end;

function AddBackSlash(aPath: string): string;
begin
   Result := EmptyStr;
   if (aPath <> '') and (aPath[Length(aPath)] <> '\') then
      Result := aPath + '\';
end;

// permet de retrouver un repertoire d'aprs sa valeur CSIDL
function GetFolder(CSIDLValue: integer): string;
var
   IdList: PITEMIDLIST;
   Folder: array[0..MAX_PATH] of char;
begin
   if not Failed(SHGetSpecialFolderLocation(0, CSIDLValue, IdList)) then begin
      SHGetPathFromIDList(IdList, Folder);
      Result := AddBackSlash(Folder);
   end;
end;

initialization
   mesDocsDir := GetFolder(CSIDL_Personal) + 'Regressi';
   if not DirectoryExists(MesDocsDir) then
      createDir(MesDocsDir);
   mesDocsDir := AddBackSlash(MesDocsDir);

   NomFichierIni := mesDocsDir+'TDS2000.ini';
   nomVisa := '';
end.

