unit Main;

interface

uses Windows, Classes, Graphics, Forms, Controls, Menus,
   Dialogs, StdCtrls, Buttons, ExtCtrls, SysUtils, Messages,
   clipBrd, spin, inifiles, Variants, math,
   jmmUtil, OleCtrls, TVCLib_TLB, ComCtrls,
   graphe;

const
   Tektro_VID= $0699;  //  1689
   TDS2000_PID= $0364;
   tds2002Visa1= 'USB0::0x0699::0x0364::C056595::INSTR';
   tds2000Visa2= 'USB0::1689::868::C056595::0::INSTR';
   tds1000Visa=  'USB0::1689::866::C060240::0::INSTR';
   VisaZero= 'Oscilloscope non trouv';


type
   EserieError = class(Exception);
   EgetFloatError = class(Exception);
   Tmode        = (mNone, mOsc, mMult, mBode);

   TFormMain = class(TForm)
      BoutonPanel: TPanel;
      OscBtn:      TBitBtn;
      ExitBtn:     TBitBtn;
      AboutBtn:    TBitBtn;
      MultBtn:     TBitBtn;
      BodeBtn:     TBitBtn;
      TekVisaGB: TGroupBox;
      TekVisaList: TComboBox;
    FindBtn: TButton;
    ConnectBtn: TButton;
    AidePanel: TStatusBar;
    Tvc1: TTvc;
      procedure ExitItemClick(Sender: TObject);
      procedure AboutItemClick(Sender: TObject);
      procedure FormCreate(Sender: TObject);
      procedure FormClose(Sender: TObject; var Action: TCloseAction);
      procedure OscBtnClick(Sender: TObject);
      procedure BodeBtnClick(Sender: TObject);
    procedure FindBtnClick(Sender: TObject);
    procedure ConnectBtnClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure MultBtnClick(Sender: TObject);
   private
      procedure AppActivate(Sender: TObject);
      procedure ShowHint(Sender: TObject);
   public
{$IFDEF Debug}
      fichierDebug : textFile;
{$ENDIF}
      IdentiteScope, NomScope: String;
      AcquisitionEnCours: Tmode;
      oscilloPresent : boolean;
      function GetString(const commande: WideString): WideString;
      function GetFloat(const commande: WideString): double;
      function GetInt(const commande: WideString; defaut: integer): integer;
      function active(v : indiceVoie) : boolean;
      procedure SetMode(Acq: Tmode);
      procedure LitVoie(aVoie : indiceVoie;
                 var valeur,valeurT : vecteurOsc;
                 var mini,maxi : double;
                 var nbrePoints : integer;var xInc : double);
(*
      procedure LitVoiePS(aVoie : indiceVoie;
                 var valeur,valeurT : vecteurOsc;
                 var mini,maxi : double;
                 var nbrePoints : integer;var xInc : double);
      procedure LitVoieTVC(Avoie : indiceVoie;
                 var valeur,valeurT : vecteurOsc;
                 var mini,maxi : double;
                 var nbrePoints : integer;var xinc : double);
      procedure LitVoieJMM(Avoie : indiceVoie;
                 var valeur,valeurT : vecteurOsc;
                 var mini,maxi : double;
                 var nbrePoints : integer;var xinc : double);
      procedure LitVoieChoix(aVoie : indiceVoie;
                 var valeur,valeurT : vecteurOsc;
                 var mini,maxi : double;
                 var nbrePoints : integer;var xInc : double);
     procedure LitVoieMaths(var valeur,valeurT : vecteurOsc;
                 var unitX,unitY : string;
                 var nbrePoints : integer;var xInc : double);
 *)
   end;

var
   FormMain:     TFormMain;

implementation

uses about, curve, tds200dde, Bode, multim;

{$R *.DFM}

function TformMain.active(v : indiceVoie) : boolean;
var reponse : wideString;
    reponseS : string;
begin
    reponse := GetString(CHactive+intToStr(v)+'?');
    reponseS := string(reponse);
    result := (pos('ON',reponseS)>0) or (pos('1',reponseS)>0);
end;

procedure TformMain.AppActivate(Sender: TObject);
begin
   if OscilloPresent then
      case AcquisitionEnCours of
         mOsc: OscilloForm.OnActivate(Sender);
         mMult: MultimetreForm.OnActivate(Sender);
         mBode: BodeForm.OnActivate(Sender);
      end;
end;

procedure TFormMain.ExitItemClick(Sender: TObject);
begin
   Close
end;

procedure TFormMain.AboutItemClick(Sender: TObject);
begin
   AboutBox.ShowModal
end;

procedure TFormMain.FormCreate(Sender: TObject);
var
   Ini: Tinifile;
begin
   oscilloPresent := false;
   AcquisitionEnCours := mNone;
   Application.OnActivate := AppActivate;
   Application.OnHint := ShowHint;
   Application.ShowHint := True;
   setMode(mNone);
   Ini := TiniFile.Create(NomFichierIni);
   nomVisa := Ini.ReadString('General', 'NomVisa', nomVisa);
   Ini.Free;
{$IFDEF Debug}
      assignFile(fichierDebug,mesDocsDir+'TDS2000.txt');
      rewrite(fichierDebug);
      writeln(fichierDebug,'TekVisa TDS2000 05/04/2012');
      writeln(fichierDebug,NomVisa);
      flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
end;

procedure TFormMain.FormActivate(Sender: TObject);
begin
  if tvc1.controlInterface=nil then begin
     showMessage('TekVisa non install');
     halt;
  end;
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
var
   Ini: TInifile;
begin
   try
      if tekVisaList.text<>VisaZero then begin
      Ini := TiniFile.Create(NomFichierIni);
      Ini.WriteString('General', 'NomVisa', tekVisaList.text);
      Ini.Free;
      end;
   except
   end;
end;

procedure TFormMain.SetMode(Acq: Tmode);
var reponse : wideString;
    reponseS : string;
begin
   if (Acq<>mNone) and (AcquisitionEnCours=mNone) then begin
      TVC1.writeString('HEAD OFF');
      TVC1.writeString('DATA:ENC ASCII');
      TVC1.writeString('DAT:STAR 1'); // depuis le dbut
      TVC1.writeString('DAT:STOP 2500'); // jusqu' la fin
   end;
   if (Acq=mBode) and
      oscilloPresent and
      (AcquisitionEnCours<>mBode) then begin
         reponse := getString('DIS:FORM?');
         reponseS := string(reponse);
         if (pos('XY',reponseS)>0) then
             TVC1.writeString('DIS:FORM YT');
   end;
   if (Acq=mOsc) and (OscilloForm=nil) then
      OscilloForm := ToscilloForm.Create(self);
   if (Acq=mBode) and (BodeForm=nil) then
      BodeForm := TBodeForm.Create(self);
 //  if (Acq=mMaths) and (MathsForm=nil) then application.CreateForm(TMathsForm,MathsForm);
   if (Acq=mMult) and (MultimetreForm=nil) then
      MultimetreForm := TmultimetreForm.Create(self);
   OscBtn.Visible  := OscilloPresent and (Acq<>mBode);
   MultBtn.Visible := (OscilloPresent) and not (Acq in [mOsc, mBode]);
   BodeBtn.Visible := OscilloPresent and not (Acq in [mOsc]);
   AcquisitionEnCours := Acq;
   case Acq of
        mOsc : OscilloForm.Show;
        mMult : MultimetreForm.Show;
        mBode :  BodeForm.Show;
      //  mMaths : MathsForm.Show;
   end;
end;

function TFormMain.GetString(const Commande: WideString): WideString;
begin
try
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'Command:#'+commande+'#');
{$ENDIF}
      TVC1.writeString(commande);
      result := tvc1.readString;
{$IFDEF Debug}
      writeln(fichierDebug,'Rponse:#'+result+'#');
{$ENDIF}
except
      result := ''
end;
{$IFDEF Debug}
      flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
end;

procedure TFormMain.MultBtnClick(Sender: TObject);
begin
 if AcquisitionEnCours=mMult
      then MultimetreForm.Show
      else setMode(mMult);
end;

function TFormMain.GetFloat(const Commande: WideString): double;
var
   ChaineLue: WideString;
begin
   try
      chaineLue := getString(commande);
      if chaineLue = '' then begin
         Result := Nan;
         exit;
      end;
// direct car Head OFF actif
      Result := StrToFloatPoint(chaineLue);
      if abs(Result) > 1e37 then Result := Nan;
   except
      Result := Nan
   end;
end;

function TFormMain.GetInt(const Commande: WideString; defaut: integer): integer;
var
   chaineLue: WideString;
begin
   try
   chaineLue := GetString(commande);
   GetInt := StrToInt(chaineLue);
   except
      GetInt := 1;
   end;
end;

procedure TFormMain.ShowHint(Sender: TObject);
begin
   if Application.Hint = '' then
      case AcquisitionEnCours of
         mOsc: OscilloForm.Caption := 'Lecture des courbes : ' + NomScope;
         //mMaths: OscilloForm.Caption := 'Lecture voie Maths : ' + NomScope;
         else AidePanel.simpleText := IdentiteScope;
      end
   else
      case AcquisitionEnCours of
         mOsc: OscilloForm.Caption := Application.Hint;
         mMult: MultimetreForm.Caption := Application.Hint;
         mBode: BodeForm.StatusBar.simpleText := Application.Hint;
         //mMaths: MathsForm.StatusBar.simpleText := Application.Hint;
         else AidePanel.simpleText := Application.Hint;
      end;
end;

procedure TFormMain.OscBtnClick(Sender: TObject);
begin
   if AcquisitionEnCours=mOsc
      then OscilloForm.Show
      else setMode(mOsc);
end;

procedure TFormMain.BodeBtnClick(Sender: TObject);
begin
   if AcquisitionEnCours=mBode
      then BodeForm.Show
      else setMode(mBode);
end;

procedure TFormMain.FindBtnClick(Sender: TObject);
var
   i : Integer;
   dev : OleVariant;
   device : string;
begin
try
  Tvc1.SearchCriterion := 0; // search all devices
  dev := Tvc1.FindList; // populate devices listbox
  tekVisalist.items.Clear;
  tekVisaList.text := '';
  If VarIsEmpty(dev) or varIsNull(dev) Then begin
     tekVisaList.text := VisaZero;
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'dev vide findbtn');
      flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
     exit;
  end;
  if VarIsArray(dev) Then begin
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'Nombre '+intTostr(VarArrayHighBound(dev,1)-VarArrayLowBound(dev,1)));
{$ENDIF}
     For i := VarArrayLowBound(dev,1) To VarArrayHighBound(dev,1) do begin
         device := varArrayGet(dev,[i]);
         tekVisalist.items.Add(device);
{$IFDEF Debug}
      writeln(fichierDebug,device);
{$ENDIF}
         If pos('USB',device)>0 then // default to USB device
            tekVisaList.itemIndex := i;
     end;
     if tekVisalist.itemIndex<0 then tekVisalist.itemIndex := 0;
{$IFDEF Debug}
      flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
  end
  else begin
     tekVisaList.text := VisaZero;
{$IFDEF Debug}
    append(fichierDebug);
    writeln(fichierDebug,'dev not array findbtn');
    flush(fichierDebug);
    closeFile(fichierDebug);
{$ENDIF}
  end;
except
  tekVisaList.text := VisaZero;
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'except findbtn');
      flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
end;
  ConnectBtn.Visible :=  tekVisaList.text<>VisaZero;
end;

procedure TFormMain.ConnectBtnClick(Sender: TObject);
const StrNoOsc = 'Oscillo non branch ou non reconnu';
var debut, fin : integer;
begin
   try
   oscilloPresent := false;
   Tvc1.Descriptor := TekVisaList.Text;
   identiteScope := GetString('*IDN?');
   if (Pos('TEK', IdentiteScope) <> 0) or
      (Pos('TDS', IdentiteScope) <> 0) then begin
         oscilloPresent := true;
// TEKTRONIX,TDS 2002B,C056595,CF:91.1CT FV:v22.11
         debut := pos(',', IdentiteScope);
         fin := pos(':', IdentiteScope);
         NomScope := copy(IdentiteScope, succ(debut), fin - debut - 3);
         Caption  := 'Acquisition avec Tektronix ' + nomScope;
   end
   else begin
{$IFDEF Debug}
      append(fichierDebug);
      writeln(fichierDebug,'descriptor='+TekVisaList.text);
      writeln(fichierDebug,'*IDN='+identiteScope);
      flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
   end;
   except
         Caption := strNoOsc;
         oscilloPresent := false;
   end;
   setMode(mNone);
end;

procedure TFormMain.litVoie(aVoie : indiceVoie;var valeur,valeurT : vecteurOsc;
   var mini,maxi : double;var nbrePoints : integer;var xInc : double);
// Paulo Santos
var
 commande,reponse : WideString;
 yinc : Double;
 pos,posVal : TStringList;
 trigpos : integer;
 i : integer;
 t,v,decaleInt,decaleVolt : Double;
begin
try
{$IFDEF Debug}
   append(fichierDebug);
   writeln(fichierDebug,'lecture de CH',avoie);
   flush(fichierDebug);
   closeFile(fichierDebug);
{$ENDIF}
   maxi := getFloat('CH'+intToStr(avoie)+':SCA?');
   maxi := 5*maxi;
   mini := -maxi;
   commande := 'DATA:SOURCE CH'+intTostr(avoie);
{$IFDEF Debug}
   append(fichierDebug);
   writeln(fichierDebug,'Mini ',mini,' ; Maxi ',maxi);
   writeln(fichierDebug,commande);
   flush(fichierDebug);
   closeFile(fichierDebug);
{$ENDIF}
   TVC1.WriteString(commande);
   reponse := GetString('CURVe?');
   pos := Tstringlist.Create;
   pos.Delimiter := ',';
   pos.DelimitedText:= reponse;
   reponse := GetString('WFMPre:CH'+intTostr(avoie)+'?');
   posVal := Tstringlist.Create;
   posVal.Delimiter := ';';
   posVal.DelimitedText := reponse;
   trigpos := strToInt(posVal[3]);
   xinc := strToFloatPoint(posval[2]);
{$IFDEF Debug}
   append(fichierDebug);
   writeln(fichierDebug,'Xinc ',posVal[2],' ; TrigPos ',posVal[3],' ; vUnit ',posVal[9],' ; hUnit ',posVal[5]);
{$ENDIF}
   NbrePoints := strToInt(posVal[10]);
   if NbrePoints>50 then begin
{$IFDEF Debug}
   writeln(fichierDebug,'NbrePoints ',NbrePoints);
{$ENDIF}
      decaleInt := strToFloatPoint(posVal[8]);   // 2.5E1 ?
      yinc := strToFloatPoint(posVal[6]);
      decaleVolt := strToFloatPoint(posVal[7]); // O.OE0 ?
      for I := 0 to NbrePoints - 1 do begin
          t := (i - trigpos) * xinc;// calculate time
          valeurT[i] := t;
          v := (strToFloat(pos[i])-decaleInt)*yinc+decaleVolt;
          valeur[i] := v;
       end;// for i
    end; // array
    pos.Free;
    posVal.Free;
except
{$IFDEF Debug}
   writeln(fichierDebug,'Erreur lecture CH',avoie);
{$ENDIF}
end;
{$IFDEF Debug}
   writeln(fichierDebug,'Fin lecture voie');
   flush(fichierDebug);
   closeFile(fichierDebug);
{$ENDIF}
end;

(*
procedure TFormMain.litVoiePS(aVoie : indiceVoie;var valeur,valeurT : vecteurOsc;
   var mini,maxi : double;var nbrePoints : integer;var xInc : double);
   // PS = Paulo Santos
var
 commande,reponse : string;
 yinc : Double;
 pos,posVal:TStringList;
 trigpos : integer;
 i : integer;
 t,v,decaleInt,decaleVolt : Double;
// Command:#SEL:CH1?#
// Rponse:#1#
// lecture de CH1
// Command:#CH1:SCAle?#
// Rponse:#2.0E-1#
// Erreur de lecture
begin
try
{$IFDEF Debug}
   append(fichierDebug);
   writeln(fichierDebug,'lecture de CH',avoie);
   flush(fichierDebug);
   closeFile(fichierDebug);
{$ENDIF}
   maxi := getFloat('CH'+intToStr(avoie)+':SCA?');
   maxi := 5*maxi;
   mini := -maxi;
   commande := 'DATA:SOURCE CH'+intTostr(avoie);
{$IFDEF Debug}
   append(fichierDebug);
   writeln(fichierDebug,'Mini ',mini,' ; Maxi ',maxi);
   writeln(fichierDebug,commande);
   flush(fichierDebug);
   closeFile(fichierDebug);
{$ENDIF}
   TVC1.WriteString(commande);
   reponse := GetString('CURVe?');
   pos := Tstringlist.Create;
   pos.Delimiter := ',';
   pos.DelimitedText:= reponse;
   reponse := GetString('WFMPre:CH'+intTostr(avoie)+'?');
   posVal := Tstringlist.Create;
   posVal.Delimiter := ';';
   posVal.DelimitedText := reponse;
   trigpos := strToInt(posVal[3]);
   xinc := strToFloatPoint(posval[2]);
{$IFDEF Debug}
   append(fichierDebug);
   writeln(fichierDebug,'Xinc ',posVal[2],' ; TrigPos ',posVal[3],' ; vUnit ',posVal[9],' ; hUnit ',posVal[5]);
{$ENDIF}
   NbrePoints := strToInt(posVal[10]);
   if NbrePoints>50 then begin
{$IFDEF Debug}
   writeln(fichierDebug,'NbrePoints ',NbrePoints);
{$ENDIF}
      decaleInt := strToFloatPoint(posVal[8]);
      yinc := strToFloatPoint(posVal[6]);
      decaleVolt := strToFloatPoint(posVal[7]);
      for I := 0 to NbrePoints - 1 do begin
          t := (i - trigpos) * xinc;// calculate time
          valeurT[i] := t;
          v := (strToFloat(pos[i])-decaleInt)*yinc+decaleVolt;
          valeur[i] := v;
       end;// for i
    end; // array
    pos.Free;
    posVal.Free;
except
{$IFDEF Debug}
   writeln(fichierDebug,'Erreur lecture CH',avoie);
{$ENDIF}
end;
{$IFDEF Debug}
   writeln(fichierDebug,'Fin lecture voie');
   flush(fichierDebug);
   closeFile(fichierDebug);
{$ENDIF}
end;
*)

(*
procedure TFormMain.litVoieMaths(var valeur,valeurT : vecteurOsc;
   var unitX,unitY : string;
   var nbrePoints : integer;var xInc : double);
   // Paulo Santos
var
 commande,reponse : string;
 yinc : Double;
 pos,posVal:TStringList;
 trigpos : integer;
 i : integer;
 t,v,decaleInt,decaleVolt : Double;
begin
try
{$IFDEF Debug}
   append(fichierDebug);
   writeln(fichierDebug,'lecture de voie Maths');
   flush(fichierDebug);
   closeFile(fichierDebug);
{$ENDIF}
   commande := 'DATA:SOURCE MATHS';
{$IFDEF Debug}
   append(fichierDebug);
   writeln(fichierDebug,commande);
   flush(fichierDebug);
   closeFile(fichierDebug);
{$ENDIF}
   TVC1.WriteString(commande);
   reponse := GetString('CURVe?');
   pos := Tstringlist.Create;
   pos.Delimiter := ',';
   pos.DelimitedText:= reponse;
   reponse := GetString('WFMPre:MATHS?');
   posVal := Tstringlist.Create;
   posVal.Delimiter := ';';
   posVal.DelimitedText := reponse;
   trigPos := strToInt(posVal[3]);
   xinc := strToFloatPoint(posval[2]);
{$IFDEF Debug}
   append(fichierDebug);
   writeln(fichierDebug,'Xinc ',posVal[2],' ; TrigPos ',posVal[3],' ; vUnit ',posVal[9],' ; hUnit ',posVal[5]);
{$ENDIF}
   NbrePoints := strToInt(posVal[10]);
   if NbrePoints>50 then begin
{$IFDEF Debug}
   writeln(fichierDebug,'NbrePoints ',NbrePoints);
{$ENDIF}
      decaleInt := strToFloatPoint(posVal[8]);
      yinc := strToFloatPoint(posVal[6]);
      unitX := posVal[5];
      unitY := posVal[9];
      decaleVolt := strToFloatPoint(posVal[7]);
      for I := 0 to NbrePoints - 1 do begin
          t := (i - trigpos) * xinc;// calculate time
          valeurT[i] := t;
          v := (strToFloat(pos[i])-decaleInt)*yinc+decaleVolt;
          valeur[i] := v;
       end;// for i
    end; // array
    pos.Free;
    posVal.Free;
except
{$IFDEF Debug}
   writeln(fichierDebug,'Erreur lecture voie Maths');
{$ENDIF}
end;
{$IFDEF Debug}
   writeln(fichierDebug,'Fin lecture voie Maths');
   flush(fichierDebug);
   closeFile(fichierDebug);
{$ENDIF}
end;
*)

(*
procedure TFormMain.LitVoieTVC(Avoie : indiceVoie;var valeur,valeurT : vecteurOsc;
    var mini,maxi : double;var nbrePoints : integer;var xInc : double);
var arrWF : OleVariant;
 trigpos : integer;
 i : integer;
 t,v : Double;
 hUnits : wideString;
 vUnits : wideString;
begin
try
{$IFDEF Debug}
   append(fichierDebug);
   writeln(fichierDebug,'lecture de CH',avoie);
{$ENDIF}
   Tvc1.GetWaveform(Avoie, arrWF, xInc, trigpos, vUnits, hUnits);
   If VarIsArray(arrWF) Then begin
      NbrePoints := VarArrayHighBound(arrWF,1) - varArrayLowBound(arrWF,1) + 1;
      For i := varArrayLowBound(arrWF,1) To varArrayHighBound(arrWF,1) do begin
          t := (i - trigpos) * xInc;
          valeurT[i] := t;
          v := varArrayGet(arrWF,[i]);
          valeur[i] := v;
{$IFDEF Debug}
      writeln(fichierDebug,i,t,v);
{$ENDIF}
       end;// for i
    end // array
    else begin
{$IFDEF Debug}
      writeln(fichierDebug,'vert=',vunits);
      writeln(fichierDebug,'horiz=',hunits);
      writeln(fichierDebug,'incX=',xInc);
{$ENDIF}
    end;
except
end;
{$IFDEF Debug}
      flush(fichierDebug);
      closeFile(fichierDebug);
{$ENDIF}
end;
*)

(*
procedure TFormMain.LitVoieJMM(Avoie : indiceVoie;var valeur,valeurT : vecteurOsc;
    var mini,maxi : double;var nbrePoints : integer;var xInc : double);
var
 arrWF : OleVariant;
 trigpos : integer;
 i : integer;
 t,v : Double;
 hUnits : wideString;
 vUnits : wideString;
begin
try
// tekVisa ; lecture de CH1
// Mini -5.00000000000000E+0000 ; Maxi  5.00000000000000E+0000
// Xinc  2.00000000000000E-0007 ; TrigPos -248750 ; vUnit Volts ; hUnit s
// NbrePoints 2500
// Fin lecture voie
   sleep(500); // ??
{$IFDEF Debug}
   append(fichierDebug);
   writeln(fichierDebug,'lecture de CH',avoie);
   flush(fichierDebug);
   closeFile(fichierDebug);
{$ENDIF}
   maxi := getFloat('CH'+intToStr(avoie)+':SCA?');
   maxi := 5*maxi;
   mini := -maxi;
{$IFDEF Debug}
   append(fichierDebug);
   writeln(fichierDebug,'Mini=',mini,' ; Maxi=',maxi);
{$ENDIF}
   Tvc1.GetWaveform(avoie, arrWF, xinc, trigpos, vUnits, hUnits);
{$IFDEF Debug}
   writeln(fichierDebug,'Xinc=',Xinc,' ; TrigPos=',trigpos,' ; vUnit=',vUnits,' ; hUnit=',hUnits);
{$ENDIF}
   If VarIsArray(arrWF) Then begin
      NbrePoints := VarArrayHighBound(arrWF,1) - varArrayLowBound(arrWF,1) + 1;
{$IFDEF Debug}
   writeln(fichierDebug,'NbrePoints ',NbrePoints);
{$ENDIF}
      For i := varArrayLowBound(arrWF,1) To varArrayHighBound(arrWF,1) do begin
          t := (i - trigpos) * xinc;// calculate time
          valeurT[i] := t;
          v := varArrayGet(arrWF,[i]);
          valeur[i] := v;
       end;// for i
    end // array
    else
{$IFDEF Debug}
   writeln(fichierDebug,'Not Array')
{$ENDIF}
    ;
except
{$IFDEF Debug}
   writeln(fichierDebug,'Erreur lecture CH',avoie);
{$ENDIF}
end;
{$IFDEF Debug}
   writeln(fichierDebug,'Fin lecture voie');
   flush(fichierDebug);
   closeFile(fichierDebug);
{$ENDIF}
end;
*)

(*
procedure TFormMain.LitVoieChoix(aVoie : indiceVoie;
                 var valeur,valeurT : vecteurOsc;
                 var mini,maxi : double;
                 var nbrePoints : integer;var xInc : double);
begin
{$IFDEF Debug}
   append(fichierDebug);
   writeln(fichierDebug,'Mthode',ord(methodeRG.itemIndex));
   flush(fichierDebug);
   closeFile(fichierDebug);
{$ENDIF}
    case MethodeRG.itemIndex of
         0 : LitVoiePS(aVoie,valeur,valeurT,mini,maxi,nbrePoints,xInc);
         1 : LitVoieTVC(aVoie,valeur,valeurT,mini,maxi,nbrePoints,xInc);
         2 : LitVoieJMM(aVoie,valeur,valeurT,mini,maxi,nbrePoints,xInc);
    end;
end;
*)

end.


